using System;
using System.Collections;
using System.ComponentModel;
using System.Web;
using System.Web.SessionState;
using System.IO;
using System.Xml;

namespace Reservation 
{
	/// <summary>
	/// Generated.
	/// Method modified: Application_BeginRequest.
	/// <p>Copyright (c) 2002 Alexis Grandemange<br/>
	/// Mail: alexis.grandemange@pagebox.net</p>
	/// <pre>This program is free software; you can redistribute it and/or
	/// modify it under the terms of the GNU Lesser General Public
	/// License as published by the Free Software Foundation; version
	/// 2.1 of the License.
	/// This library is distributed in the hope that it will be useful,
	/// but WITHOUT ANY WARRANTY; without even the implied warranty of
	/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	/// GNU Lesser General Public License for more details.
	/// A copy of the GNU Lesser General Public License lesser.txt should be
	/// included in the distribution.</pre>
	/// </summary>
	public class Global : System.Web.HttpApplication
	{
		/// <summary>
		/// Database connection string
		/// </summary>
		public static string connectionString = null;
		/// <summary>
		/// URL of PageBox Query.asmx
		/// </summary>
		public static string pageboxUrl = null;
		/// <summary>
		/// true when initialized
		/// </summary>
		public static bool isInitialized = false;
		/// <summary>
		/// Reservation archive name
		/// </summary>
		public static string archive;
		/// <summary>
		/// SMTP server used to send confirmation messages (Register.aspx and Retrieve.aspx)
		/// </summary>
		public static string smtpServer;
		/// <summary>
		/// Configuration user
		/// </summary>
		public static string user;
		/// <summary>
		/// Configuration password
		/// </summary>
		public static string password;
		/// <summary>
		/// Country codes per country names
		/// </summary>
		public static Hashtable countries;
		/// <summary>
		/// US State codes per state names
		/// </summary>
		public static Hashtable states;
		/// <summary>
		/// Canada Province codes per province names
		/// </summary>
		public static Hashtable provinces;
		/// <summary>
		/// Country names per country codes
		/// </summary>
		public static Hashtable countryPerCode = null;
		/// <summary>
		/// US State names per state codes
		/// </summary>
		public static Hashtable statePerCode = null;
		/// <summary>
		/// Canada Province names per province codes
		/// </summary>
		public static Hashtable provincePerCode = null;
		/// <summary>
		/// Sorted array of state + province names
		/// </summary>
		public static ArrayList stateAl = new ArrayList();
		/// <summary>
		/// State or province name
		/// </summary>
		public static string stateName = null;
		/// <summary>
		/// Index of stateName in stateAl
		/// </summary>
		public static int stateIndex = 0;
		/// <summary>
		/// Sorted array of country names
		/// </summary>
		public static ArrayList countryAl = new ArrayList();
		/// <summary>
		/// Country name
		/// </summary>
		public static string countryName = null;
		/// <summary>
		/// Index of countryName in countryAl
		/// </summary>
		public static int countryIndex = 0;
		/// <summary>
		/// Reservation instance address
		/// </summary>
		public static string address;
		public static string city;
		/// <summary>
		/// State or province code
		/// </summary>
		public static string state;
		public static string zipcode;
		/// <summary>
		/// Country code
		/// </summary>
		public static string country;
		public static string region;
		public static string latitude;
		public static string longitude;
		/// <summary>
		/// URL of the directory of this Reservation instance
		/// </summary>
		public static string url;
		/// <summary>
		/// Application full physical path
		/// </summary>
		public static string path;

		protected void Application_Start(Object sender, EventArgs e)
		{
		}
 
		protected void Session_Start(Object sender, EventArgs e)
		{
		}

		/// <summary>
		/// Computes the physical path and the URL.
		/// Restores reservation.xml.
		/// </summary>
		protected void Application_BeginRequest(Object sender, EventArgs e)
		{
			lock(typeof(Global)) 
			{
				if (!Global.isInitialized) 
				{
					if (path == null) 
					{	// Very first time
						path = Request.ServerVariables["PATH_TRANSLATED"];
						int pos = path.LastIndexOf("\\");
						path = path.Substring(0, pos + 1);
						string lpath = Request.ServerVariables["PATH_INFO"];
						string host = Request.ServerVariables["HTTP_HOST"];
						string port = Request.ServerVariables["SERVER_PORT"];
						pos = lpath.LastIndexOf("/");
						lpath = lpath.Substring(0, pos);
						if (port.Equals("80"))
							url = host + lpath;
						else
							url = host + ":" + port + lpath;
					}
					if (File.Exists(path + "reservation.xml")) 
					{
						XmlTextReader r = new XmlTextReader(path + "reservation.xml");
						r.WhitespaceHandling = WhitespaceHandling.None;
						r.ReadStartElement("reservation");
						Global.archive = r.ReadElementString("archive");
						Global.connectionString = r.ReadElementString("connection-string");
						Global.pageboxUrl = r.ReadElementString("pagebox");
						Global.smtpServer = r.ReadElementString("smtp-server");
						Global.user = r.ReadElementString("user");
						Global.password = r.ReadElementString("password");
						Global.address = r.ReadElementString("address");
						Global.city = r.ReadElementString("city");
						Global.state = r.ReadElementString("state");
						Global.zipcode = r.ReadElementString("zip");
						Global.country = r.ReadElementString("country");
						Global.region = r.ReadElementString("region");
						Global.latitude = r.ReadElementString("latitude");
						Global.longitude = r.ReadElementString("longitude");
						r.Close();
						if (File.Exists(path + "countries.csv"))
							Global.countries = restoreCsv(path + "countries.csv", 2);
						if (File.Exists(path + "states.csv"))
							Global.states = restoreCsv(path + "states.csv", 0);
						if (File.Exists(path + "provinces.csv"))
							Global.provinces = restoreCsv(path + "provinces.csv", 1);
						Global.stateAl.Add("N/A");
						ArrayList stateAl = new ArrayList(Global.states.Keys);
						stateAl.Sort();
						ArrayList provinceAl = new ArrayList(Global.provinces.Keys);
						provinceAl.Sort();
						if (country.Equals("CA")) 
						{
							Global.stateAl.AddRange(provinceAl);
							Global.stateAl.AddRange(stateAl);
						} 
						else 
						{
							Global.stateAl.AddRange(stateAl);
							Global.stateAl.AddRange(provinceAl);
						}
						if ((Global.state != null) && (Global.state.Length > 0)) 
						{
							if (Global.statePerCode.Contains(Global.state))
								Global.stateName = (string)Global.statePerCode[Global.state];
							else 
							{
								if (Global.provincePerCode.Contains(Global.state))
									Global.stateName = (string)Global.provincePerCode[Global.state];
							}
						}
						if (Global.stateName != null) 
						{
							for (int i = 0; i < Global.stateAl.Count; ++ i) 
							{
								string st = (string)Global.stateAl[i];
								if (st.Equals(Global.stateName)) 
								{
									Global.stateIndex = i;
									break;
								}
							}
						}
						Global.countryAl = new ArrayList(Global.countries.Keys);
						Global.countryAl.Sort();
						Global.countryName = (string)Global.countryPerCode[Global.country];
						for (int i = 0; i < Global.countryAl.Count; ++ i) 
						{
							string c = (string)Global.countryAl[i];
							if (c.Equals(Global.countryName)) 
							{
								Global.countryIndex = i;
								break;
							}
						}
						Global.isInitialized = true;
					}
				}
			}
		}

		/// <summary>
		/// Populates a Hashtable from a csv file.
		/// </summary>
		/// <param name="csvp">Path of a csv file val, key</param>
		/// <param name="hpc">Type of csv</param>
		/// <returns>Populated Hashtable key=name</returns>
		private static Hashtable restoreCsv(string csvp, int type) 
		{
			Hashtable h = new Hashtable();
			Hashtable hpc = new Hashtable();
			StreamReader sr = new StreamReader(csvp);
			string line = null;
			while((line = sr.ReadLine()) != null) 
			{
				string[] la = line.Split(new char[] { ',' });
				if (la.Length >= 2) 
				{
					h.Add(la[1], la[0]);
					hpc.Add(la[0], la[1]);
				}
			}
			sr.Close();
			switch(type) 
			{
				case 0:
					Global.statePerCode = hpc;
					break;
				case 1:
					Global.provincePerCode = hpc;
					break;
				case 2:
					Global.countryPerCode = hpc;
					break;
			}
			return h;
		}

		protected void Application_EndRequest(Object sender, EventArgs e)
		{

		}

		protected void Session_End(Object sender, EventArgs e)
		{

		}

		protected void Application_End(Object sender, EventArgs e)
		{

		}
	}
}

